package com.persagy.account.manage;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Lists;
import com.persagy.account.pojo.dto.SaasFunction;
import com.persagy.account.pojo.dto.SaasGroupProduct;
import com.persagy.account.pojo.dto.SaasMenu;
import com.persagy.account.pojo.dto.SaasProductPage;
import com.persagy.account.pojo.dto.SaasProjectProduct;
import com.persagy.account.pojo.dto.SaasRoleMenu;
import com.persagy.account.service.ISaasFunctionService;
import com.persagy.account.service.ISaasGroupProductService;
import com.persagy.account.service.ISaasProductPageService;
import com.persagy.account.service.ISaasProjectProductService;
import com.persagy.account.service.ISaasRoleMenuService;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.exception.BusinessException;
import com.persagy.common.utils.StringUtil;

import cn.hutool.core.collection.CollectionUtil;

/**
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021年5月6日 上午10:26:15
 */
@Component
public class SaasProductHandler {
	
    @Autowired
    private ISaasFunctionService saasFunctionService;
    
    @Autowired
    private ISaasRoleMenuService saasRoleMenuService;
    
	@Autowired
	private ISaasProductPageService saasProductPageService;
	
    @Autowired
    private ISaasGroupProductService saasGroupProductService;
    
    @Autowired
    private ISaasProjectProductService saasProjectProductService;

    
	/**
	 * 这里未签约，不会返回异常，会设置为空的集合
	 * 获取此账号下能够可见的页面集合，优先做权限判断，判定是否进行了签约
	 * 
	 * @param saasMenu 不可为空
	 * @param accountId
	 * @param projectId
	 * @param groupCode
	 * @param isAdmin
	 * @param showFunctions
	 */
	public void queryMenuProductAuths(SaasMenu saasMenu, String accountId, String projectId, String groupCode, boolean isAdmin, boolean showFunctions) {
		// angle 1-集团视角，2-项目视角
		String angle = StringUtil.isBlank(projectId) ? SaasCommonConstant.STR_STATUS_1 : SaasCommonConstant.STR_STATUS_2;
		if (StringUtil.isBlank(saasMenu.getProductId())) {
			List<SaasFunction> functions = null;
			// 未挂有产品，关联查询出对应的功能点集合，即可返回
			if (isAdmin) {
				LambdaQueryWrapper<SaasFunction> queryWrapper = new SaasFunction.Builder().createQueryWrapper().menuIdEq(saasMenu.getMenuId()).builderQueryWrapper();
				functions = this.saasFunctionService.list(queryWrapper.orderByAsc(SaasFunction::getCreateTime));
			} else {
				functions = this.saasFunctionService.queryAccountFunctionList(accountId, saasMenu.getGroupCode(), saasMenu.getMenuId());
			}
			saasMenu.setFunctions(CollectionUtil.isEmpty(functions) ? Lists.newArrayList() : functions);
			return;
		}
		// 挂有产品，判断是否已签约该产品
		int count = 0;
		if (SaasCommonConstant.STR_STATUS_2.equals(angle)) {
			LambdaQueryWrapper<SaasProjectProduct> queryWrapper = new SaasProjectProduct.BuilderQueryWrapper().groupCodeEq(groupCode).projectIdEq(projectId).productIdEq(saasMenu.getProductId()).builder();
			count = this.saasProjectProductService.count(queryWrapper);
		} else {
			LambdaQueryWrapper<SaasGroupProduct> queryWrapper = new SaasGroupProduct.BuilderQueryWrapper().groupCodeEq(groupCode).productIdEq(saasMenu.getProductId()).builder();
			count = this.saasGroupProductService.count(queryWrapper);
		}
		if (count < 1) {
			saasMenu.setProductPages(Lists.newArrayList());
			return;	// 未签约
		}
		// 已签约，走关联查询获取页面及其权限
		List<SaasProductPage> pageTree = this.querySaasProductPageTree(angle, saasMenu.getProductId(), accountId, saasMenu.getMenuId(), isAdmin, showFunctions);
		saasMenu.setProductPages(pageTree);
		return;
	}
	
	/**
	 * 这里未签约，不会返回异常，会设置为空的集合
	 * 获取此账号下能够可见的页面集合，优先做权限判断，判定是否进行了签约
	 * 
	 * @param menuId
	 * @param productId
	 * @param accountId
	 * @param projectId
	 * @param groupCode
	 * @param isAdmin
	 * @param showPageAuth
	 */
	/*public Object queryMenuProductAuths(String menuId, String productId, String accountId, String projectId, String groupCode, boolean isAdmin, boolean showPageAuth) {
		// angle 1-集团视角，2-项目视角
		String angle = StringUtil.isBlank(projectId) ? SaasCommonConstant.STR_STATUS_1 : SaasCommonConstant.STR_STATUS_2;
		
		if (StringUtil.isBlank(productId)) {
			// 未挂有产品，关联查询出对应的功能点集合，即可返回
			List<SaasFunction> functions = null;
			if (isAdmin) {
				LambdaQueryWrapper<SaasFunction> queryWrapper = new SaasFunction.Builder().createQueryWrapper().menuIdEq(menuId).builderQueryWrapper();
				functions = this.saasFunctionService.list(queryWrapper.orderByAsc(SaasFunction::getCreateTime));
			} else {
				functions = this.saasFunctionService.queryAccountFunctionList(accountId, groupCode, menuId);
			}
			functions = CollectionUtil.isEmpty(functions) ? Lists.newArrayList() : functions;
			return functions;
		}
		// 挂有产品，判断是否已签约该产品
		int count = 0;
		if (SaasCommonConstant.STR_STATUS_2.equals(angle)) {
			LambdaQueryWrapper<SaasProjectProduct> queryWrapper = new SaasProjectProduct.BuilderQueryWrapper().groupCodeEq(groupCode).projectIdEq(projectId).productIdEq(productId).builder();
			count = this.saasProjectProductService.count(queryWrapper);
		} else {
			LambdaQueryWrapper<SaasGroupProduct> queryWrapper = new SaasGroupProduct.BuilderQueryWrapper().groupCodeEq(groupCode).productIdEq(productId).builder();
			count = this.saasGroupProductService.count(queryWrapper);
		}
		if (count < 1) {
			return Lists.newArrayList();	// 未签约
		}
		// 已签约，走关联查询获取页面及其权限
		List<SaasProductPage> pageTree = this.querySaasProductPageTree(angle, productId, accountId, menuId, isAdmin, showPageAuth);
		return pageTree;
	}*/
	
	/**
	 * 获取产品页面树
	 * @param angle
	 * @param productId
	 * @param accountId
	 * @param menuId
	 * @param isAdmin 当为超管时，accountId、menuId可为空
	 * @param showPageAuth
	 * @return 空集合或数据集合
	 */
	public List<SaasProductPage> querySaasProductPageTree(String angle, String productId, String accountId, String menuId, boolean isAdmin, boolean showPageAuth) {
		// 查询出该产品下顶级页面信息集合
		List<SaasProductPage> topList = this.getTopProductPageList(angle, productId);
		if (topList.size() == 0) {
			return topList;
		}
		this.querySaasProductPageTree(topList, accountId, menuId, angle, isAdmin, showPageAuth);
		return topList;
	}
	
	/**
	 * 获取顶级产品页面信息集合
	 * @param angle
	 * @param productId
	 * @return
	 */
	public List<SaasProductPage> getTopProductPageList(String angle, String productId) {
		LambdaQueryWrapper<SaasProductPage> queryWrapper = new SaasProductPage.BuilderQueryWrapper().pageAngleIn(angle).productIdEq(productId).builder();
		queryWrapper.apply("(PARENT_ID IS NULL OR TRIM(PARENT_ID) = '')");
		queryWrapper.orderByAsc(SaasProductPage::getCreateTime);
		List<SaasProductPage> topList = this.saasProductPageService.list(queryWrapper);
		return CollectionUtil.isEmpty(topList) ? Lists.newArrayList() : topList;
	}
	
	/**
	 * 封装产品页面树
	 * @param topList
	 * @param accountId
	 * @param menuId
	 * @param angle 当前视角
	 * @param isAdmin 是否为超管，true-是，超管时，accountId、menuId均可为空
	 * @param showPageAuth true-查询出对应页面的权限点信息
	 */
	private void querySaasProductPageTree(List<SaasProductPage> topList, String accountId, String menuId, String angle, boolean isAdmin, boolean showPageAuth) {
		for (SaasProductPage saasProductPage : topList) {
			List<SaasProductPage> nextList = null;
			if (isAdmin) {
				nextList = this.getAdminProductPageAuth(saasProductPage, angle, showPageAuth);
			} else {
				nextList = this.getCommonProductPageAuth(saasProductPage, accountId, menuId, angle, showPageAuth);
			}
			saasProductPage.setChildrens(nextList);
			this.querySaasProductPageTree(nextList, accountId, menuId, angle, isAdmin, showPageAuth);
		}
	}
	
	/**
	 * 获取超管对应的产品页面权限
	 * 
	 * @param saasProductPage
	 * @param angle
	 * @param showPageAuth
	 * @return 空集合
	 */
	private List<SaasProductPage> getAdminProductPageAuth(SaasProductPage saasProductPage, String angle, boolean showPageAuth) {
		if (showPageAuth) {
			// 获取此页面的权限集合
			LambdaQueryWrapper<SaasFunction> queryWrapper = new SaasFunction.Builder().createQueryWrapper().pageIdEq(saasProductPage.getId()).builderQueryWrapper();
			List<SaasFunction> functions = this.saasFunctionService.list(queryWrapper);
			saasProductPage.setFunctions(functions);
		}
		// 获取下级页面
		LambdaQueryWrapper<SaasProductPage> queryWrapper = new SaasProductPage.BuilderQueryWrapper().pageAngleIn(angle).parentIdEq(saasProductPage.getId()).builder();
		List<SaasProductPage> list = this.saasProductPageService.list(queryWrapper);
		return CollectionUtil.isEmpty(list) ? Lists.newArrayList() : list;
	}
	
	/**
	 * 获取普通用户对应的产品页面权限
	 * 
	 * @param saasProductPage
	 * @param accountId;
	 * @param menuId
	 * @param angle
	 * @param showPageAuth
	 * @return 空集合
	 */
	private List<SaasProductPage> getCommonProductPageAuth(SaasProductPage saasProductPage, String accountId, String menuId, String angle, boolean showPageAuth) {
		if (showPageAuth) {
			// 获取此页面的权限集合，走表关联查询
			List<SaasFunction> functions = this.saasFunctionService.queryProductPageFunctions(accountId, menuId, saasProductPage.getId());
			saasProductPage.setFunctions(functions);
		}
		// 获取下级页面，走表关联查询
		List<SaasProductPage> list = this.saasProductPageService.queryAccountProductPages(accountId, saasProductPage.getProductId(), menuId, saasProductPage.getId(), angle);
		return CollectionUtil.isEmpty(list) ? Lists.newArrayList() : list;
	}

	/**
	 * 级联删除所有的子菜单和对应的权限信息
	 * @param menuId
	 * @param productId
	 */
	@Transactional
	public void casDeleteChildrenPage(String parentId) {
		LambdaQueryWrapper<SaasProductPage> queryWrapper = new SaasProductPage.BuilderQueryWrapper().parentIdEq(parentId).builder();
		List<SaasProductPage> productPages = this.saasProductPageService.list(queryWrapper.select(SaasProductPage::getId, SaasProductPage::getPageFloor));
		if (CollectionUtil.isEmpty(productPages)) {
			return;
		}
		for (SaasProductPage productPage : productPages) {
			// 删除此页面
			if (!SaasCommonConstant.STR_STATUS_0.equals(productPage.getPageFloor())) {
				throw new BusinessException("落地页不允许删除，请优先变更落地页所属");
			}
	    	this.deleteSaasProductPage(productPage.getId());
		}
	}
	
	/**
	 * 删除产品页面信息，级联删除页面和子页面下权限,并与其权限相关的关联关系
	 * @param pageId
	 * @return
	 */
	@Transactional
	public boolean deleteSaasProductPage(String pageId) {
		if (StringUtil.isBlank(pageId)) {
			return true;
		}
		boolean result = this.saasProductPageService.removeById(pageId);
		if (result) {
			// 1.删除权限点信息
			LambdaQueryWrapper<SaasFunction> funWrapper = new SaasFunction.Builder().createQueryWrapper().pageIdEq(pageId).builderQueryWrapper();
			this.saasFunctionService.remove(funWrapper);
			// 2.删除此页面对应的菜单、角色关系数据
			LambdaQueryWrapper<SaasRoleMenu> roleMenuWrapper = new SaasRoleMenu.Builder().createQueryWrapper().pageIdEq(pageId).builderQueryWrapper();
			this.saasRoleMenuService.remove(roleMenuWrapper);
			this.casDeleteChildrenPage(pageId);
			return true;
		}
		throw new BusinessException("产品页面信息删除失败");
	}

}
