package com.persagy.account.pojo.dto;

import java.util.Date;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.persagy.common.annotation.IdGeneratorPrefix;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.model.BaseEntity;
import com.persagy.common.utils.StringUtil;

import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

/**
 * 功能信息
 * 
 * @version 1.0.0
 * @company persagy
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Getter
@Setter
@ToString
@EqualsAndHashCode(callSuper = false)
@TableName("saas_function")
@IdGeneratorPrefix(SaasCommonConstant.SAAS_FUNCTION_ID_PREFIX)
public class SaasFunction extends BaseEntity<SaasFunction> {
	private static final long serialVersionUID = 7760376088679533302L;

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String menuId; // 菜单ID
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String pageId;	// 页面ID

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String funCode; // 功能编码

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String funName; // 功能名称
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String funUrl;	//功能URL

	@TableField(fill = FieldFill.INSERT, updateStrategy = FieldStrategy.NOT_EMPTY)
	private Date createTime; // 创建时间

	@TableField(fill = FieldFill.INSERT_UPDATE, updateStrategy = FieldStrategy.NOT_EMPTY)
	private Date updateTime; // 更新时间

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String updateUser; // 最后一次操作者ID

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String remark; // 备注

	@TableField(exist = false)
	private String groupCode; // 集团编码
	
	public static class Builder {

		private LambdaQueryWrapper<SaasFunction> queryWrapper = null;

		private LambdaUpdateWrapper<SaasFunction> updateWrapper = null;

		public Builder createQueryWrapper() {
			queryWrapper = new LambdaQueryWrapper<SaasFunction>();
			return this;
		}

		public Builder createUpdateWrapper() {
			updateWrapper = new LambdaUpdateWrapper<SaasFunction>();
			return this;
		}

		public Builder idEq(String id) {
			if (StringUtil.isNotBlank(id)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasFunction::getId, id);
				} else {
					queryWrapper.eq(SaasFunction::getId, id);
				}
			}
			return this;
		}

		public Builder menuIdEq(String menuId) {
			if (StringUtil.isNotBlank(menuId)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasFunction::getMenuId, menuId);
				} else {
					queryWrapper.eq(SaasFunction::getMenuId, menuId);
				}
			}
			return this;
		}
		
		public Builder pageIdEq(String pageId) {
			if (StringUtil.isNotBlank(pageId)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasFunction::getPageId, pageId);
				} else {
					queryWrapper.eq(SaasFunction::getPageId, pageId);
				}
			}
			return this;
		}

		public Builder funCodeEq(String funCode) {
			if (StringUtil.isNotBlank(funCode)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasFunction::getFunCode, funCode);
				} else {
					queryWrapper.eq(SaasFunction::getFunCode, funCode);
				}
			}
			return this;
		}

		public Builder funNameEq(String funName) {
			if (StringUtil.isNotBlank(funName)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasFunction::getFunName, funName);
				} else {
					queryWrapper.eq(SaasFunction::getFunName, funName);
				}
			}
			return this;
		}
		
		public Builder funNameLike(String funName) {
			if (StringUtil.isNotBlank(funName)) {
				if (updateWrapper != null) {
					updateWrapper.like(SaasFunction::getFunName, funName);
				} else {
					queryWrapper.like(SaasFunction::getFunName, funName);
				}
			}
			return this;
		}
		
		public Builder funUrlEq(String funUrl) {
			if (StringUtil.isNotBlank(funUrl)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasFunction::getFunUrl, funUrl);
				} else {
					queryWrapper.eq(SaasFunction::getFunUrl, funUrl);
				}
			}
			return this;
		}

		public Builder createTimeEq(Date createTime) {
			if (null != createTime) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasFunction::getCreateTime, createTime);
				} else {
					queryWrapper.eq(SaasFunction::getCreateTime, createTime);
				}
			}
			return this;
		}

		public Builder updateTimeEq(Date updateTime) {
			if (null != updateTime) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasFunction::getUpdateTime, updateTime);
				} else {
					queryWrapper.eq(SaasFunction::getUpdateTime, updateTime);
				}
			}
			return this;
		}

		public Builder updateUserEq(String updateUser) {
			if (StringUtil.isNotBlank(updateUser)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasFunction::getUpdateUser, updateUser);
				} else {
					queryWrapper.eq(SaasFunction::getUpdateUser, updateUser);
				}
			}
			return this;
		}

		public Builder remarkEq(String remark) {
			if (StringUtil.isNotBlank(remark)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasFunction::getRemark, remark);
				} else {
					queryWrapper.eq(SaasFunction::getRemark, remark);
				}
			}
			return this;
		}

		public LambdaQueryWrapper<SaasFunction> builderQueryWrapper() {
			return queryWrapper;
		}

		public LambdaUpdateWrapper<SaasFunction> builderUpdateWrapper() {
			return updateWrapper;
		}
	}
}
