package com.persagy.account.pojo.dto;

import java.util.Date;
import java.util.List;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.persagy.common.annotation.IdGeneratorPrefix;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.model.BaseEntity;
import com.persagy.common.utils.StringUtil;

import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

/**
 * 产品信息
 * 
 * @version 1.0.0
 * @company persagy
 * @author zhangqiankun
 * @date 2021-04-27 16:08:25
 */
@Getter
@Setter
@ToString
@EqualsAndHashCode(callSuper = false)
@TableName("saas_product")
@IdGeneratorPrefix(SaasCommonConstant.SAAS_PRODUCT_ID_PREFIX)
public class SaasProduct extends BaseEntity<SaasProduct> {
	private static final long serialVersionUID = -7374882065014346250L;

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String appId; // 应用ID， PC-电脑端，APP-手机端
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String productCode; // 产品编码

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String productName; // 产品名称

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String productIcon; // 产品图标

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String productState; // 产品状态，0-建设中，1-已完成，2-已关联

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String productMenu; // 产品是否关联菜单，0-未关联，1-已关联

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String productType;	//产品类型，0-系统产品，1-标准业务产品，2-IBMS产品
	
	@TableField(fill = FieldFill.INSERT, updateStrategy = FieldStrategy.NOT_EMPTY)
	private Date createTime; // 创建时间

	@TableField(fill = FieldFill.INSERT_UPDATE, updateStrategy = FieldStrategy.NOT_EMPTY)
	private Date updateTime; // 更新时间

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String updateUser; // 最后一次操作者ID

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String remark; 		// 备注
	
	@TableField(exist = false)
	private List<SaasProductPage> productPages; 		// 下级所有页面集合
	
	public static class BuilderQueryWrapper {

		private LambdaQueryWrapper<SaasProduct> queryWrapper = new LambdaQueryWrapper<SaasProduct>();

		public BuilderQueryWrapper idEq(String id) {
			if (StringUtil.isNotBlank(id)) {
				queryWrapper.eq(SaasProduct::getId, id);
			}
			return this;
		}
		
		public BuilderQueryWrapper appIdEq(String appId) {
			if (StringUtil.isNotBlank(appId)) {
				queryWrapper.eq(SaasProduct::getAppId, appId);
			}
			return this;
		}

		public BuilderQueryWrapper productCodeEq(String productCode) {
			if (StringUtil.isNotBlank(productCode)) {
				queryWrapper.eq(SaasProduct::getProductCode, productCode);
			}
			return this;
		}

		public BuilderQueryWrapper productNameEq(String productName) {
			if (StringUtil.isNotBlank(productName)) {
				queryWrapper.eq(SaasProduct::getProductName, productName);
			}
			return this;
		}
		
		public BuilderQueryWrapper productNameLike(String productName) {
			if (StringUtil.isNotBlank(productName)) {
				queryWrapper.like(SaasProduct::getProductName, productName);
			}
			return this;
		}

		public BuilderQueryWrapper productIconEq(String productIcon) {
			if (StringUtil.isNotBlank(productIcon)) {
				queryWrapper.eq(SaasProduct::getProductIcon, productIcon);
			}
			return this;
		}

		public BuilderQueryWrapper productStateEq(String productState) {
			if (StringUtil.isNotBlank(productState)) {
				queryWrapper.eq(SaasProduct::getProductState, productState);
			}
			return this;
		}

		public BuilderQueryWrapper productMenuEq(String productMenu) {
			if (StringUtil.isNotBlank(productMenu)) {
				queryWrapper.eq(SaasProduct::getProductMenu, productMenu);
			}
			return this;
		}
		
		public BuilderQueryWrapper productTypeEq(String productType) {
			if (StringUtil.isNotBlank(productType)) {
				queryWrapper.eq(SaasProduct::getProductType, productType);
			}
			return this;
		}

		public BuilderQueryWrapper createTimeEq(Date createTime) {
			if (null != createTime) {
				queryWrapper.eq(SaasProduct::getCreateTime, createTime);
			}
			return this;
		}

		public BuilderQueryWrapper updateTimeEq(Date updateTime) {
			if (null != updateTime) {
				queryWrapper.eq(SaasProduct::getUpdateTime, updateTime);
			}
			return this;
		}

		public BuilderQueryWrapper updateUserEq(String updateUser) {
			if (StringUtil.isNotBlank(updateUser)) {
				queryWrapper.eq(SaasProduct::getUpdateUser, updateUser);
			}
			return this;
		}

		public BuilderQueryWrapper remarkEq(String remark) {
			if (StringUtil.isNotBlank(remark)) {
				queryWrapper.eq(SaasProduct::getRemark, remark);
			}
			return this;
		}

		public LambdaQueryWrapper<SaasProduct> builder() {
			return queryWrapper;
		}

	}

	public static class BuilderUpdateWrapper {

		private LambdaUpdateWrapper<SaasProduct> updateWrapper = new LambdaUpdateWrapper<SaasProduct>();

		public BuilderUpdateWrapper idEq(String id) {
			if (StringUtil.isNotBlank(id)) {
				updateWrapper.eq(SaasProduct::getId, id);
			}
			return this;
		}
		
		public BuilderUpdateWrapper appIdEq(String appId) {
			if (StringUtil.isNotBlank(appId)) {
				updateWrapper.eq(SaasProduct::getAppId, appId);
			}
			return this;
		}

		public BuilderUpdateWrapper productCodeEq(String productCode) {
			if (StringUtil.isNotBlank(productCode)) {
				updateWrapper.eq(SaasProduct::getProductCode, productCode);
			}
			return this;
		}

		public BuilderUpdateWrapper productNameEq(String productName) {
			if (StringUtil.isNotBlank(productName)) {
				updateWrapper.eq(SaasProduct::getProductName, productName);
			}
			return this;
		}

		public BuilderUpdateWrapper productIconEq(String productIcon) {
			if (StringUtil.isNotBlank(productIcon)) {
				updateWrapper.eq(SaasProduct::getProductIcon, productIcon);
			}
			return this;
		}

		public BuilderUpdateWrapper productStateEq(String productState) {
			if (StringUtil.isNotBlank(productState)) {
				updateWrapper.eq(SaasProduct::getProductState, productState);
			}
			return this;
		}

		public BuilderUpdateWrapper productMenuEq(String productMenu) {
			if (StringUtil.isNotBlank(productMenu)) {
				updateWrapper.eq(SaasProduct::getProductMenu, productMenu);
			}
			return this;
		}
		
		public BuilderUpdateWrapper productTypeEq(String productType) {
			if (StringUtil.isNotBlank(productType)) {
				updateWrapper.eq(SaasProduct::getProductType, productType);
			}
			return this;
		}

		public BuilderUpdateWrapper createTimeEq(Date createTime) {
			if (null != createTime) {
				updateWrapper.eq(SaasProduct::getCreateTime, createTime);
			}
			return this;
		}

		public BuilderUpdateWrapper updateTimeEq(Date updateTime) {
			if (null != updateTime) {
				updateWrapper.eq(SaasProduct::getUpdateTime, updateTime);
			}
			return this;
		}

		public BuilderUpdateWrapper updateUserEq(String updateUser) {
			if (StringUtil.isNotBlank(updateUser)) {
				updateWrapper.eq(SaasProduct::getUpdateUser, updateUser);
			}
			return this;
		}

		public BuilderUpdateWrapper remarkEq(String remark) {
			if (StringUtil.isNotBlank(remark)) {
				updateWrapper.eq(SaasProduct::getRemark, remark);
			}
			return this;
		}

		public LambdaUpdateWrapper<SaasProduct> builder() {
			return updateWrapper;
		}
	
	}
	
}
