package com.persagy.account.pojo.vo.product;

import java.util.Date;
import java.util.Set;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;

import com.persagy.common.model.BaseGroupModel;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

/**
 * 产品签约信息
 * 
 * @version 1.0.0
 * @company persagy
 * @author zhangqiankun
 * @date 2021-05-07 10:31:47
 */
@Getter
@Setter
@ApiModel
@ToString
public class SaasGroupProductCreateVO extends BaseGroupModel {
	
	@ApiModelProperty(value = "签约ID，更新时不可为空", example = "1001")
	private String id;
	
	@NotBlank(message = "产品ID，不可为空")
	@ApiModelProperty(value = "产品ID", example = "1001", required = true)
	private String productId; // 产品ID

	@NotBlank(message = "产品使用范围，不可为空")
	@ApiModelProperty(value = "产品使用范围，0-全部项目，1-部分项目", example = "0", required = true)
	private String productRange; // 产品使用范围，0-全部项目，1-部分项目
	
	@NotEmpty(message = "页面ID集合，不可为空")
	@ApiModelProperty(value = "页面ID集合", example = "PA1001")
	private Set<String> pageIds; // 页面ID集合
	
	@ApiModelProperty(value = "签约有效期的开始时间", example = "6456465165")
	private Date validStartTime; // 签约有效期的开始时间

	@ApiModelProperty(value = "签约有效期的结束时间", example = "6456465165")
	private Date validEndTime; // 签约有效期的结束时间

	@NotBlank(message = "签约有效期，不可为空")
	@ApiModelProperty(value = "签约有效期，0-期内有效，1-永久有效", example = "0")
	private String validLast; // 签约有效期，0-期内有效，1-永久有效
	
	@ApiModelProperty(value = "备注", example = "测试")
	private String remark; // 备注

}
