package com.persagy.account.service;

import java.util.List;

import com.persagy.common.service.ISuperService;
import com.persagy.account.pojo.dto.SaasMenu;
import com.persagy.account.pojo.vo.menu.SaasMenuSort;

/**
 * 菜单信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
public interface ISaasMenuService extends ISuperService<SaasMenu> {

	/**
	 * 获取账号菜单集合
	 * @param accountId
	 * @param groupCode
	 * @param appId
	 * @param menuId 可为null
	 * @param valid
	 * @return
	 */
	List<SaasMenu> querySaasMenuTree(String accountId, String groupCode, String appId, String menuId, Integer valid);

	/**
	 * 查询出此角色下，所有不存在功能点的菜单集合
	 * @param groupCode
	 * @param roleId
	 * @return
	 */
	List<SaasMenu> queryMenuAndEmptyFunction(String groupCode, String roleId);

	/**
	 * 批量更新菜单的顺序
	 * 
	 * @param menus
	 * @return
	 */
	boolean sortSaasMenu(List<SaasMenuSort> menus);

	/**
	 * 验证菜单编码在应用下的唯一性
	 * 
	 * @param appId
	 * @param menuType
	 * @param menuCode
	 * @param menuId
	 * @return
	 */
	boolean validMenuCode(String appId, String menuType, String menuCode, String menuId);
	
	/**
	 * 验证菜单名称的唯一性
	 * 
	 * @param appId
	 * @param menuType
	 * @param menuName
	 * @param menuId
	 * @return
	 */
	boolean validMenuName(String appId, String menuType, String menuName, String menuId);
	
	/**
	 * 创建菜单，同时设置产品是否已关联菜单
	 * @param saasMenu
	 * @return
	 */
	boolean createSaasMenu(SaasMenu saasMenu);

	/**
	 * 更新菜单
	 * 
	 * @param saasMenu
	 * @param orgProductId 原纪录的产品ID 
	 * @return
	 */
	boolean updateSaasMenu(SaasMenu saasMenu, String orgProductId);

}

