package com.persagy.account.service;

import java.util.List;

import com.persagy.account.pojo.dto.SaasProductPage;
import com.persagy.common.service.ISuperService;

/**
 * 产品页面配置
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-05-06 10:12:47
 */
public interface ISaasProductPageService extends ISuperService<SaasProductPage> {

	/**
	 * 获取集团所签约的页面集合
	 * 
	 * @param groupCode
	 * @param productId
	 * @param angle
	 * @return
	 */
	List<SaasProductPage> queryGroupProductPages(String groupCode, String productId, String angle);
	
	/**
	 * 获取项目所签约的此产品的页面集合
	 * 
	 * @param groupCode
	 * @param projectId
	 * @param productId
	 * @param angle
	 * @return
	 */
	List<SaasProductPage> queryProjectProductPages(String groupCode, String projectId, String productId, String angle);

	/**
	 * 获取此账号所拥有的页面权限
	 * @param accountId
	 * @param productId
	 * @param menuId
	 * @param parentId
	 * @param angle
	 * @return
	 */
	List<SaasProductPage> queryAccountProductPages(String accountId, String productId, String menuId, String parentId, String angle);

	/**
	 * 创建产品页面
	 * @param saasProductPage
	 * @return
	 */
	boolean createSaasProductPage(SaasProductPage saasProductPage);

	/**
	 * 更新产品页面
	 * @param saasProductPage
	 * @return
	 */
	boolean updateSaasProductPage(SaasProductPage saasProductPage);

	/**
	 * 验证页面名称在此页面下的唯一性
	 * @param productId
	 * @param pageName
	 * @param pageId
	 * @return
	 */
	boolean validPageName(String productId, String pageName, String pageId);

}

