package com.persagy.account.service;

import java.util.List;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.persagy.common.service.ISuperService;
import com.persagy.account.pojo.dto.SaasProject;

/**
 * 项目信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
public interface ISaasProjectService extends ISuperService<SaasProject> {

	/**
	 * 查询 项目信息
	 * @param groupCode
	 * @param projectId
	 * @return
	 */
	SaasProject getOne(String groupCode, String projectId);
	
	/**
	 * 获取项目集合信息，关联查询
	 * 
	 * @param saasProject
	 * @param page 当前页，可以为空
	 * @param size 每页大小，可以为空
	 * @return records不会为null
	 */
	Page<SaasProject> queryMoreProjectInfo(SaasProject saasProject, Integer page, Integer size);
	
	/**
	 * 项目列表查询
	 * 
	 * @param saasProject
	 * @param page 当前页，可以为空
	 * @param size 每页大小，可以为空
	 * @return records不会为null
	 */
	Page<SaasProject> querySaasProjectList(SaasProject saasProject, Integer page, Integer size);
	
	/**
	 * 查询该账户可允许访问的集团项目，权限表关联查询
	 * 
	 * @param groupCode
	 * @param accountId
	 * @return
	 */
	List<SaasProject> queryAllowGroupProjectList(String groupCode, String accountId);
	
	/**
	 * 验证项目ID、项目本地编码、项目名称，分别在集团下的唯一性
	 * 
	 * @param saasProject
	 * @param isUpdate，true时，id不可为空
	 * @return true-唯一，false-已存在
	 */
	boolean validProjectData(SaasProject saasProject, boolean isUpdate);

	/**
	 * 批量插入项目信息
	 * 
	 * @param list
	 * @return
	 */
	boolean batchInsertProjectData(List<SaasProject> list);

	/**
	 * 批量更新项目所属区域
	 * 
	 * @param saasProject
	 * @param projectIds
	 * @return
	 */
	boolean batchUpdateProjectArea(SaasProject saasProject, List<String> projectIds);

	/**
	 * 查询出能够被访问的项目总数
	 * 
	 * @param groupCode 集团编码
	 * @param accountId 账号ID
	 * @param areaId 底层区域ID
	 * @return
	 */
	int countAllowProjectNum(String groupCode, String accountId, String areaId);

}

