package com.persagy.account.service;

import java.util.List;

import com.persagy.common.service.ISuperService;
import com.persagy.account.pojo.dto.SaasRoleMenu;
import com.persagy.account.pojo.vo.auth.SaasMenuFunctionPageVO;
import com.persagy.account.pojo.vo.auth.SaasMenuFunctionVO;

/**
 * 角色-菜单功能关联信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
public interface ISaasRoleMenuService extends ISuperService<SaasRoleMenu> {

	/**
	 * 批量创建角色对应的菜单功能权限
	 * @param groupCode
	 * @param roleId
	 * @param auths
	 * @return
	 */
	boolean batchCreateSaasRoleMenu(String groupCode, String roleId, List<SaasMenuFunctionVO> auths);

	/**
	 * 批量创建角色对应的菜单页面功能权限
	 * @param groupCode
	 * @param roleId
	 * @param auths
	 * @return
	 */
	boolean batchCreateSaasRoleMenuPage(String groupCode, String roleId, List<SaasMenuFunctionPageVO> auths);
	
	/**
	 * 验证是否允许删除菜单
	 * @param groupCode
	 * @param menuId
	 * @return true-允许
	 */
	boolean isAllowDeleteMenu(String groupCode, String menuId);

	/**
	 * 获取此角色下菜单数量
	 * 
	 * @param roleId
	 * @return
	 */
	int countMenu(String roleId);
	
	/**
	 * 获取以被分配的功能点数量
	 * 
	 * @param roleId
	 * @param menuId
	 */
	int countFunction(String roleId, String menuId);

}

