package com.persagy.account.service.impl;

import java.util.List;

import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.service.impl.SuperServiceImpl;
import com.persagy.common.utils.StringUtil;
import com.persagy.account.mapper.SaasGroupMapper;
import com.persagy.account.pojo.dto.SaasGroup;
import com.persagy.account.service.ISaasGroupService;

/**
 * 集团信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Service
public class SaasGroupServiceImpl extends SuperServiceImpl<SaasGroupMapper, SaasGroup> implements ISaasGroupService {

	@Override
	public SaasGroup getOne(String groupCode) {
		LambdaQueryWrapper<SaasGroup> queryWrapper = new SaasGroup.Builder().createQueryWrapper()
				.groupCodeEq(groupCode).validEq(SaasCommonConstant.STATUS_1).builderQueryWrapper();
		return this.baseMapper.selectOne(queryWrapper);
	}
	
	@Override
	public boolean validGroupCodeOrName(String groupCode, String groupName, boolean isUpdate) {
		LambdaQueryWrapper<SaasGroup> queryWrapper = null;
		if (StringUtil.isNotBlank(groupCode)) {
			queryWrapper = new SaasGroup.Builder().createQueryWrapper().groupCodeEq(groupCode)
					.validEq(SaasCommonConstant.STATUS_1).builderQueryWrapper();
			SaasGroup selectOne = this.baseMapper.selectOne(queryWrapper);
			if (selectOne != null && !isUpdate) {
				return false;
			}
		}
		if (StringUtil.isNotBlank(groupName)) {
			queryWrapper = new SaasGroup.Builder().createQueryWrapper().groupNameEq(groupName)
					.validEq(SaasCommonConstant.STATUS_1).builderQueryWrapper();
			SaasGroup selectOne = this.baseMapper.selectOne(queryWrapper);
			if (selectOne != null && !isUpdate) {
				return false;
			}
			if (isUpdate && selectOne != null && !groupCode.equals(selectOne.getGroupCode())) {
				return false;
			}
		}
		return true;
	}

	@Override
	public List<SaasGroup> queryAllowGroupInfo(SaasGroup saasGroup, String accountId) {
		return this.baseMapper.queryAllowGroupInfo(saasGroup, accountId);
	}

	@Override
	public List<SaasGroup> queryGroupAndEmptyProject(String groupCode, String accountId) {
		return this.baseMapper.queryGroupAndEmptyProject(groupCode, accountId);
	}

}
