package com.persagy.account.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import com.persagy.account.mapper.SaasMenuMapper;
import com.persagy.account.mapper.SaasProductMapper;
import com.persagy.account.pojo.dto.SaasMenu;
import com.persagy.account.pojo.dto.SaasProduct;
import com.persagy.account.pojo.vo.menu.SaasMenuSort;
import com.persagy.account.service.ISaasMenuService;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.service.impl.SuperServiceImpl;
import com.persagy.common.utils.StringUtil;

/**
 * 菜单信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Service
public class SaasMenuServiceImpl extends SuperServiceImpl<SaasMenuMapper, SaasMenu> implements ISaasMenuService {
	
	@Autowired
	private SaasProductMapper saasProductMapper;
	
	@Override
	public List<SaasMenu> querySaasMenuTree(String accountId, String groupCode, String appId, String menuId, Integer valid) {
		return this.baseMapper.querySaasMenuTree(accountId, groupCode, appId, menuId, valid);
	}

	@Override
	public List<SaasMenu> queryMenuAndEmptyFunction(String groupCode, String roleId) {
		return this.baseMapper.queryMenuAndEmptyFunction(groupCode, roleId);
	}

	@Override
	@Transactional
	public boolean sortSaasMenu(List<SaasMenuSort> menus) {
		Integer result = this.baseMapper.sortSaasMenu(menus);
		return SqlHelper.retBool(result);
	}

	@Override
	public boolean validMenuCode(String appId, String menuType, String menuCode, String menuId) {
		LambdaQueryWrapper<SaasMenu> queryWrapper = new SaasMenu.Builder().createQueryWrapper().appIdEq(appId).menuTypeEq(menuType).menuCodeEq(menuCode).builderQueryWrapper();
		SaasMenu selectOne = this.baseMapper.selectOne(queryWrapper);
		if (selectOne != null && (StringUtil.isBlank(menuId) || !menuId.equals(selectOne.getId()))) {
			return false;
		}
		return true;
	}
	
	@Override
	public boolean validMenuName(String appId, String menuType, String menuName, String menuId) {
		LambdaQueryWrapper<SaasMenu> queryWrapper = new SaasMenu.Builder().createQueryWrapper().appIdEq(appId).menuTypeEq(menuType).menuNameEq(menuName).builderQueryWrapper();
		SaasMenu selectOne = this.baseMapper.selectOne(queryWrapper);
		if (selectOne != null && (StringUtil.isBlank(menuId) || !menuId.equals(selectOne.getId()))) {
			return false;
		}
		return true;
	}

	@Override
	@Transactional
	public boolean createSaasMenu(SaasMenu saasMenu) {
		Integer result = this.baseMapper.insert(saasMenu);
		if (SqlHelper.retBool(result) && StringUtil.isNotBlank(saasMenu.getProductId())) {
			// 更新产品是否已关联菜单字段
			SaasProduct saasProduct = new SaasProduct();
			saasProduct.setId(saasMenu.getProductId());
			saasProduct.setProductMenu(SaasCommonConstant.STR_STATUS_1);
			result = this.saasProductMapper.updateById(saasProduct);
		}
		return SqlHelper.retBool(result);
	}

	@Override
	@Transactional
	public boolean updateSaasMenu(SaasMenu saasMenu, String orgProductId) {
		Integer result = null;
		if (StringUtil.isAllBlank(saasMenu.getProductId(), orgProductId)) {
			result = this.baseMapper.updateById(saasMenu);
			return SqlHelper.retBool(result);
		}
		if (StringUtil.isNotBlank(orgProductId) && StringUtil.isBlank(saasMenu.getProductId())) {
			// 判断此产品是否仅被此菜单使用
    		LambdaQueryWrapper<SaasMenu> queryWrapper = new SaasMenu.Builder().createQueryWrapper().productId(saasMenu.getProductId()).builderQueryWrapper();
    		int count = this.baseMapper.selectCount(queryWrapper);
    		if (count < 2) {
    			// 更新产品是否已关联菜单字段
				SaasProduct saasProduct = new SaasProduct();
				saasProduct.setId(orgProductId);
				saasProduct.setProductMenu(SaasCommonConstant.STR_STATUS_0);
				this.saasProductMapper.updateById(saasProduct);
			}
    		// 更新此菜单信息
    		LambdaUpdateWrapper<SaasMenu> updateWrapper = new SaasMenu.Builder().idEq(saasMenu.getId()).builderUpdateWrapper();
    		updateWrapper.setSql("PRODUCT_ID = NULL");
			result = this.baseMapper.update(saasMenu, updateWrapper);
			return SqlHelper.retBool(result);
		}
		// 更新产品是否已关联菜单字段
		result = this.baseMapper.updateById(saasMenu);
		if (SqlHelper.retBool(result)) {
			SaasProduct saasProduct = new SaasProduct();
			saasProduct.setId(saasMenu.getProductId());
			saasProduct.setProductMenu(SaasCommonConstant.STR_STATUS_1);
			result = this.saasProductMapper.updateById(saasProduct);
		}
		return SqlHelper.retBool(result);
	}

}
