package com.persagy.sso.client;

import com.persagy.common.utils.ResponseResult;
import com.persagy.sso.client.fallback.FmPersonClientFallbackFactory;
import com.persagy.sso.config.FeignConfig;
import com.persagy.sso.constant.FeignClientNameConstant;
import com.persagy.sso.pojo.dto.*;
import feign.Headers;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;

/**
 * @Description: 调用fm-person的接口信息
 * @Author: xgq
 * @Date: 2021/4/14 21:51
 * @return: null
 **/
@FeignClient(name=FeignClientNameConstant.FM_PERSON_CLIENT_NAME,
        url=FeignClientNameConstant.FM_PERSON_CLIENT_URL , fallbackFactory = FmPersonClientFallbackFactory.class,
        configuration = FeignConfig.class)
public interface FmPersonClient {

    /**
     * @Description: 新建部门
     * @Author: xgq
     * @Date: 2021/4/14 21:59
     * @param addDepartmentDTO :
     * @param token
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/fm-person/departments/create")
    ResponseResult saveDept(@RequestBody AddDepartmentDTO addDepartmentDTO, @RequestHeader("token") String token);

    /**
     * @Description: 删除部门
     * @Author: xgq
     * @Date: 2021/4/14 22:09
     * @param idDTO:
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/fm-person/departments/delete")
    ResponseResult deleteDept(@RequestBody IdDTO idDTO);


    /**
     * @Description: 创建岗位角色
     * @Author: xgq
     * @Date: 2021/4/14 22:09
     * @param addJobDTO :
     * @param token
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/fm-person/saasRoles/create")
    ResponseResult saveJob(@RequestBody AddJobDTO addJobDTO,@RequestHeader("token") String token);


    /**
     * @Description: 删除岗位角色
     * @Author: xgq
     * @Date: 2021/4/14 22:09
     * @param idDTO:
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/fm-person/saasRoles/delete")
    ResponseResult deleteJob(@RequestBody IdDTO idDTO);

    /**
     * @Description: 保存人员信息
     * @Author: xgq
     * @Date: 2021/4/14 22:09
     * @param addPersonDTO :
     * @param token
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/fm-person/persons/sync_create")
    ResponseResult savePerson(@RequestBody AddPersonDTO addPersonDTO,@RequestHeader("token") String token);


    /**
     * @Description: 根据id查询部门信息
     * @Author: xgq
     * @Date: 2021/4/14 22:09
     * @param idDTO:
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/fm-person/departments/get")
    ResponseResult queryDeptById(@RequestBody IdDTO idDTO);

    /**
     * @Description: 更新部门信息
     * @Author: xgq
     * @Date: 2021/4/15 1:43
     * @param addDepartmentDTO :
     * @param token
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/fm-person/departments/update")
    ResponseResult updateDept(@RequestBody AddDepartmentDTO addDepartmentDTO, @RequestHeader("token") String token);



    /**
     * @Description: 更新岗位信息
     * @Author: xgq
     * @Date: 2021/4/15 1:51
     * @param addJobDTO :
     * @param token
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/fm-person/saasRoles/update")
    ResponseResult updateJob(@RequestBody AddJobDTO addJobDTO,@RequestHeader("token") String token);

    /**
     * @Description: 更新人员
     * @Author: xgq
     * @Date: 2021/4/15 2:04
     * @param addPersonDTO :
     * @param token
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/fm-person/persons/update")
    ResponseResult updatePerson(@RequestBody AddPersonDTO addPersonDTO, @RequestHeader("token") String token);


    /**
     * @Description: 根据条件查询部门信息
     * @Author: xgq
     * @Date: 2021/4/15 19:56
     * @param queryDeptDTO :
     * @param token
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/fm-person/departments/query")
    ResponseResult queryGroupDept(@RequestBody QueryDeptDTO queryDeptDTO, @RequestHeader("token") String token);

    /**
     * @Description: 根据条件查询人员
     * @Author: xgq
     * @Date: 2021/4/16 12:06
     * @param queryPersonDTO :
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/fm-person/persons/queryByUsername")
    ResponseResult queryPersonByCondition(@RequestBody QueryPersonDTO queryPersonDTO, @RequestHeader("token") String token);
}
